<?php

namespace EventPortal\Domain\ActionList\Models;

use Carbon\Carbon;
use EventPortal\Domain\User\Models\User;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Support\Arr;

class Task extends Model
{
    protected $fillable = [
        'label',
        'slug',
        'completed_at',
        'user_id',
    ];

    public function actionList(): BelongsTo
    {
        return $this->belongsTo(ActionList::class);
    }

    public function getIsCompleteAttribute(): bool
    {
        return Arr::get($this->attributes, 'completed_at') !== null;
    }

    public function getCompletedByAttribute(): ?User
    {
        return User::whereId(Arr::get($this->attributes, 'user_id'))->first();
    }

    /**
     * Toggles the complete status and returns the new completed_at state
     */
    public function toggle(User $user): bool
    {
        if ($this->isComplete) {
            return $this->uncomplete();
        }

        return $this->complete($user);
    }

    public function complete(?User $user): bool
    {
        $this->update([
            'completed_at' => Carbon::now(),
            'user_id' => optional($user)->id,
        ]);

        return true;
    }

    public function uncomplete(): bool
    {
        $this->update([
            'completed_at' => null,
            'user_id' => null,
        ]);

        return false;
    }

    public function scopeCompleted(Builder $query): Builder
    {
        return $query->whereNotNull('completed_at');
    }

    public function scopeIncompleted(Builder $query): Builder
    {
        return $query->whereNull('completed_at');
    }
}
