<?php

namespace EventPortal\Domain\Delegate\Models;

use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Model;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;

class DelegateFormData extends Model implements AuditableInterface
{
    use Auditable;

    const KEY = 'delegate_form_data';

    /** @var array<string> */
    public $with = [
        'delegate',
        'question',
        'step',
    ];

    /** @var array<string> */
    protected $fillable = [
        'delegate_id',
        'form_step_id',
        'question_id',
        'value',
    ];

    public function delegate(): BelongsTo
    {
        return $this->belongsTo(Delegate::class);
    }

    public function question(): BelongsTo
    {
        return $this->belongsTo(Question::class);
    }

    public function step(): BelongsTo
    {
        return $this->belongsTo(FormStep::class, 'form_step_id');
    }

    public function scopeForDelegate(Builder $query, Delegate $delegate): Builder
    {
        return $query->where('delegate_id', $delegate->id);
    }

    public function scopeForStep(Builder $query, FormStep $step): Builder
    {
        return $query->where('form_step_id', $step->id);
    }
}
