<?php

namespace EventPortal\Tests\Feature\Admin\Event;

use Carbon\Carbon;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Form\Models\FormStepSection;
use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Event\Form\Models\QuestionOption;
use EventPortal\Domain\Event\Form\Models\QuestionValidation;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class EditFormTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots, SeedRolesAndPermissions;

    /** @test **/
    public function can_edit_a_form(): void
    {
        Carbon::setTestNow('01/01/2020');
        $event = factory(Event::class)->create();
        $form = factory(Form::class)->state('snapshot')->create(['event_id' => $event->id]);
        $step = factory(FormStep::class)->state('snapshot')->create(['form_id' => $form->id]);
        $section = factory(FormStepSection::class)->state('snapshot')->create(['form_step_id' => $step->id]);
        $question = factory(Question::class)->state('snapshot')->create(['form_step_section_id' => $section->id]);
        factory(QuestionOption::class)->state('snapshot')->create(['question_id' => $question->id]);
        factory(QuestionValidation::class)->create(['question_id' => $question->id]);

        $this->assertMatchesJsonSnapshot(
            $this
                ->actingAsSuperAdmin()
                ->get(route('admin.form.edit', ['event' => $event->id, 'form' => $form->id]))
                ->props('form')
        );
    }
}
